@@ -111,7 +111,9 @@ struct _PhotosApplication
   GSimpleAction *set_ss_action;
   GSimpleAction *share_action;
   GSimpleAction *sharpen_action;
+  GSimpleAction *zoom_begin_action;
   GSimpleAction *zoom_best_fit_action;
+  GSimpleAction *zoom_end_action;
   GSimpleAction *zoom_in_action;
   GSimpleAction *zoom_out_action;
   GtkWidget *main_window;
@@ -158,6 +160,29 @@ static const GOptionEntry COMMAND_LINE_OPTIONS[] =
   { NULL }
 };
 
+static const gchar *REQUIRED_GEGL_OPS[] =
+{
+  "gegl:buffer-sink",
+  "gegl:buffer-source",
+  "gegl:crop",
+  "gegl:exposure",
+  "gegl:gray",
+  "gegl:load",
+  "gegl:noise-reduction",
+  "gegl:nop",
+  "gegl:pixbuf",
+  "gegl:rotate-on-center",
+  "gegl:save-pixbuf",
+  "gegl:scale-ratio",
+  "gegl:unsharp-mask",
+
+  /* Used by gegl:load */
+  "gegl:jpg-load",
+  "gegl:png-load",
+  "gegl:raw-load",
+  "gegl:text"
+};
+
 static const gchar *DESKTOP_BACKGROUND_SCHEMA = "org.gnome.desktop.background";
 static const gchar *DESKTOP_SCREENSAVER_SCHEMA = "org.gnome.desktop.screensaver";
 static const gchar *DESKTOP_KEY_PICTURE_URI = "picture-uri";
@@ -185,6 +210,7 @@ struct _PhotosApplicationRefreshData
 
 struct _PhotosApplicationSetBackgroundData
 {
+  PhotosApplication *application;
   GFile *file;
   GSettings *settings;
 };
@@ -244,11 +270,13 @@ photos_application_refresh_data_free (PhotosApplicationRefreshData *data)
 
 
 static PhotosApplicationSetBackgroundData *
-photos_application_set_background_data_new (GFile *file, GSettings *settings)
+photos_application_set_background_data_new (PhotosApplication *application, GFile *file, GSettings *settings)
 {
   PhotosApplicationSetBackgroundData *data;
 
   data = g_slice_new0 (PhotosApplicationSetBackgroundData);
+  g_application_hold (G_APPLICATION (application));
+  data->application = application;
   data->file = g_object_ref (file);
   data->settings = g_object_ref (settings);
   return data;
@@ -258,6 +286,7 @@ photos_application_set_background_data_new (GFile *file, GSettings *settings)
 static void
 photos_application_set_background_data_free (PhotosApplicationSetBackgroundData *data)
 {
+  g_application_release (G_APPLICATION (data->application));
   g_object_unref (data->file);
   g_object_unref (data->settings);
   g_slice_free (PhotosApplicationSetBackgroundData, data);
@@ -348,6 +377,7 @@ photos_application_actions_update (PhotosApplication *self)
   selection_mode = photos_utils_get_selection_mode ();
 
   g_simple_action_set_enabled (self->zoom_best_fit_action, FALSE);
+  g_simple_action_set_enabled (self->zoom_end_action, FALSE);
   g_simple_action_set_enabled (self->zoom_out_action, FALSE);
 
   enable = (mode == PHOTOS_WINDOW_MODE_EDIT);
@@ -362,7 +392,8 @@ photos_application_actions_update (PhotosApplication *self)
   g_simple_action_set_enabled (self->sharpen_action, enable);
 
   enable = FALSE;
-  if (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+  if (mode == PHOTOS_WINDOW_MODE_COLLECTION_VIEW
+      || mode == PHOTOS_WINDOW_MODE_COLLECTIONS
       || mode == PHOTOS_WINDOW_MODE_FAVORITES
       || mode == PHOTOS_WINDOW_MODE_OVERVIEW
       || mode == PHOTOS_WINDOW_MODE_SEARCH)
@@ -391,6 +422,7 @@ photos_application_actions_update (PhotosApplication *self)
   g_simple_action_set_enabled (self->gear_action, enable);
   g_simple_action_set_enabled (self->set_bg_action, enable);
   g_simple_action_set_enabled (self->set_ss_action, enable);
+  g_simple_action_set_enabled (self->zoom_begin_action, enable);
   g_simple_action_set_enabled (self->zoom_in_action, enable);
 
   enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW)
@@ -600,6 +632,32 @@ photos_application_gegl_init_fishes_idle (gpointer user_data)
 }
 
 
+static gboolean
+photos_application_sanity_check_gegl (PhotosApplication *self)
+{
+  GeglConfig *config;
+  gboolean ret_val = TRUE;
+  gboolean use_opencl;
+  guint i;
+
+  config = gegl_config ();
+  g_object_get (config, "use-opencl", &use_opencl, NULL);
+  photos_debug (PHOTOS_DEBUG_GEGL, "Using OpenCL: %s", use_opencl ? "yes" : "no");
+
+  for (i = 0; i < G_N_ELEMENTS (REQUIRED_GEGL_OPS); i++)
+    {
+      if (!gegl_has_operation (REQUIRED_GEGL_OPS[i]))
+        {
+          g_warning ("Unable to find GEGL operation %s: Check your GEGL install", REQUIRED_GEGL_OPS[i]);
+          ret_val = FALSE;
+          break;
+        }
+    }
+
+  return ret_val;
+}
+
+
 static void
 photos_application_tracker_set_rdf_types (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -653,22 +711,22 @@ photos_application_tracker_extract_priority (GObject *source_object, GAsyncResul
 static gboolean
 photos_application_create_window (PhotosApplication *self)
 {
-  gboolean gegl_sanity_checked;
   gboolean gexiv2_initialized;
   gboolean gexiv2_registered_namespace;
+  gboolean sanity_checked_gegl;
 
   if (self->main_window != NULL)
     return TRUE;
 
-  gegl_sanity_checked = photos_gegl_sanity_check ();
-  g_return_val_if_fail (gegl_sanity_checked, FALSE);
-
   gexiv2_initialized = gexiv2_initialize ();
   g_return_val_if_fail (gexiv2_initialized, FALSE);
 
   gexiv2_registered_namespace = gexiv2_metadata_register_xmp_namespace ("http://www.gnome.org/xmp", "gnome");
   g_return_val_if_fail (gexiv2_registered_namespace, FALSE);
 
+  sanity_checked_gegl = photos_application_sanity_check_gegl (self);
+  g_return_val_if_fail (sanity_checked_gegl, FALSE);
+
   self->main_window = photos_main_window_new (GTK_APPLICATION (self));
   g_signal_connect_object (self->main_window,
                            "delete-event",
@@ -1376,11 +1434,9 @@ photos_application_set_bg_common_save_to_file (GObject *source_object, GAsyncRes
 
 
 static void
-photos_application_set_bg_common (PhotosApplication *self, GVariant *parameter, gpointer user_data)
+photos_application_set_bg_common (PhotosApplication *self, GSettings *settings)
 {
   GFile *backgrounds_file = NULL;
-  GSimpleAction *action = G_SIMPLE_ACTION (user_data);
-  GSettings *settings;
   PhotosApplicationSetBackgroundData *data;
   PhotosBaseItem *item;
   const gchar *config_dir;
@@ -1410,9 +1466,7 @@ photos_application_set_bg_common (PhotosApplication *self, GVariant *parameter,
   backgrounds_path = g_build_filename (backgrounds_dir, backgrounds_filename, NULL);
   backgrounds_file = g_file_new_for_path (backgrounds_path);
 
-  settings = G_SETTINGS (g_object_get_data (G_OBJECT (action), "settings"));
-
-  data = photos_application_set_background_data_new (backgrounds_file, settings);
+  data = photos_application_set_background_data_new (self, backgrounds_file, settings);
   photos_base_item_save_to_file_async (item,
                                        backgrounds_file,
                                        G_FILE_CREATE_PRIVATE | G_FILE_CREATE_REPLACE_DESTINATION,
@@ -1429,14 +1483,32 @@ photos_application_set_bg_common (PhotosApplication *self, GVariant *parameter,
 }
 
 
+static void
+photos_application_set_background (PhotosApplication *self)
+{
+  photos_application_set_bg_common (self, self->bg_settings);
+}
+
+
+static void
+photos_application_set_screensaver (PhotosApplication *self)
+{
+  photos_application_set_bg_common (self, self->ss_settings);
+}
+
+
 static void
 photos_application_share_share (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  PhotosApplication *self;
   PhotosSharePoint *share_point = PHOTOS_SHARE_POINT (source_object);
   GError *error = NULL;
+  PhotosBaseItem *item = PHOTOS_BASE_ITEM (user_data);
+  gchar *uri = NULL;
 
-  photos_share_point_share_finish (share_point, res, &error);
+  self = PHOTOS_APPLICATION (g_application_get_default ());
+
+  photos_share_point_share_finish (share_point, res, &uri, &error);
   if (error != NULL)
     {
       g_warning ("Unable to share the image: %s", error->message);
@@ -1445,9 +1517,14 @@ photos_application_share_share (GObject *source_object, GAsyncResult *res, gpoin
       goto out;
     }
 
+  if (photos_share_point_needs_notification (share_point))
+    photos_share_notification_new (share_point, item, uri);
+
  out:
   g_application_unmark_busy (G_APPLICATION (self));
   g_application_release (G_APPLICATION (self));
+  g_free (uri);
+  g_object_unref (item);
 }
 
 
@@ -1473,7 +1550,7 @@ photos_application_share_response (GtkDialog *dialog, gint response_id, gpointer
 
   g_application_hold (G_APPLICATION (self));
   g_application_mark_busy (G_APPLICATION (self));
-  photos_share_point_share_async (share_point, item, NULL, photos_application_share_share, self);
+  photos_share_point_share_async (share_point, item, NULL, photos_application_share_share, g_object_ref (item));
 
  out:
   gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -1589,56 +1666,9 @@ photos_application_theme_changed (GtkSettings *settings)
 
 
 static void
-photos_application_window_mode_changed (PhotosApplication *self, PhotosWindowMode mode, PhotosWindowMode old_mode)
+photos_application_window_mode_changed (PhotosApplication *self)
 {
-  PhotosBaseManager *item_mngr_chld;
-
-  g_return_if_fail (mode != PHOTOS_WINDOW_MODE_NONE);
-
-  switch (old_mode)
-    {
-    case PHOTOS_WINDOW_MODE_NONE:
-    case PHOTOS_WINDOW_MODE_EDIT:
-    case PHOTOS_WINDOW_MODE_PREVIEW:
-      break;
-
-    case PHOTOS_WINDOW_MODE_COLLECTIONS:
-    case PHOTOS_WINDOW_MODE_FAVORITES:
-    case PHOTOS_WINDOW_MODE_OVERVIEW:
-    case PHOTOS_WINDOW_MODE_SEARCH:
-      item_mngr_chld = photos_item_manager_get_for_mode (PHOTOS_ITEM_MANAGER (self->state->item_mngr), old_mode);
-      g_signal_handlers_disconnect_by_func (item_mngr_chld, photos_application_items_changed, self);
-      break;
-
-    default:
-      g_assert_not_reached ();
-      break;
-    }
-
   photos_application_actions_update (self);
-
-  switch (mode)
-    {
-    case PHOTOS_WINDOW_MODE_COLLECTIONS:
-    case PHOTOS_WINDOW_MODE_FAVORITES:
-    case PHOTOS_WINDOW_MODE_OVERVIEW:
-    case PHOTOS_WINDOW_MODE_SEARCH:
-      item_mngr_chld = photos_item_manager_get_for_mode (PHOTOS_ITEM_MANAGER (self->state->item_mngr), mode);
-      g_signal_connect_swapped (item_mngr_chld,
-                                "items-changed",
-                                G_CALLBACK (photos_application_items_changed),
-                                self);
-      break;
-
-    case PHOTOS_WINDOW_MODE_EDIT:
-    case PHOTOS_WINDOW_MODE_PREVIEW:
-      break;
-
-    case PHOTOS_WINDOW_MODE_NONE:
-    default:
-      g_assert_not_reached ();
-      break;
-    }
 }
 
 
@@ -1836,12 +1866,13 @@ photos_application_startup (GApplication *application)
   const gchar *zoom_best_fit_accels[3] = {"<Primary>0", NULL};
   const gchar *zoom_in_accels[3] = {"<Primary>plus", "<Primary>equal", NULL};
   const gchar *zoom_out_accels[2] = {"<Primary>minus", NULL};
+  gchar *detailed_action_name = NULL;
 
   photos_debug (PHOTOS_DEBUG_APPLICATION, "PhotosApplication::startup");
 
   G_APPLICATION_CLASS (photos_application_parent_class)->startup (application);
 
-  photos_gegl_init ();
+  gegl_init (NULL, NULL);
 
   grl_init (NULL, NULL);
   registry = grl_registry_get_default ();
@@ -2028,19 +2059,11 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->selection_mode_action));
 
   self->set_bg_action = g_simple_action_new ("set-background", NULL);
-  g_object_set_data_full (G_OBJECT (self->set_bg_action),
-                          "settings",
-                          g_object_ref (self->bg_settings),
-                          g_object_unref);
-  g_signal_connect_swapped (self->set_bg_action, "activate", G_CALLBACK (photos_application_set_bg_common), self);
+  g_signal_connect_swapped (self->set_bg_action, "activate", G_CALLBACK (photos_application_set_background), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->set_bg_action));
 
   self->set_ss_action = g_simple_action_new ("set-screensaver", NULL);
-  g_object_set_data_full (G_OBJECT (self->set_ss_action),
-                          "settings",
-                          g_object_ref (self->ss_settings),
-                          g_object_unref);
-  g_signal_connect_swapped (self->set_ss_action, "activate", G_CALLBACK (photos_application_set_bg_common), self);
+  g_signal_connect_swapped (self->set_ss_action, "activate", G_CALLBACK (photos_application_set_screensaver), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->set_ss_action));
 
   self->share_action = g_simple_action_new ("share-current", NULL);
@@ -2050,13 +2073,19 @@ photos_application_startup (GApplication *application)
   self->sharpen_action = g_simple_action_new ("sharpen-current", G_VARIANT_TYPE_DOUBLE);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->sharpen_action));
 
+  self->zoom_begin_action = g_simple_action_new ("zoom-begin", G_VARIANT_TYPE_VARDICT);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_begin_action));
+
   self->zoom_best_fit_action = g_simple_action_new ("zoom-best-fit", NULL);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_best_fit_action));
 
-  self->zoom_in_action = g_simple_action_new ("zoom-in", G_VARIANT_TYPE_DOUBLE);
+  self->zoom_end_action = g_simple_action_new ("zoom-end", G_VARIANT_TYPE_VARDICT);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_end_action));
+
+  self->zoom_in_action = g_simple_action_new ("zoom-in", G_VARIANT_TYPE_VARDICT);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_in_action));
 
-  self->zoom_out_action = g_simple_action_new ("zoom-out", G_VARIANT_TYPE_DOUBLE);
+  self->zoom_out_action = g_simple_action_new ("zoom-out", G_VARIANT_TYPE_VARDICT);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_out_action));
 
   g_signal_connect_swapped (self->state->mode_cntrlr,
@@ -2080,9 +2109,23 @@ photos_application_startup (GApplication *application)
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.search", search_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.select-all", select_all_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.zoom-best-fit", zoom_best_fit_accels);
-  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.zoom-in(-1.0)", zoom_in_accels);
-  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.zoom-out(-1.0)", zoom_out_accels);
 
+  detailed_action_name = photos_utils_print_zoom_action_detailed_name ("app.zoom-in",
+                                                                       1.0,
+                                                                       PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), detailed_action_name, zoom_in_accels);
+  g_free (detailed_action_name);
+
+  detailed_action_name = photos_utils_print_zoom_action_detailed_name ("app.zoom-out",
+                                                                       1.0,
+                                                                       PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), detailed_action_name, zoom_out_accels);
+  g_free (detailed_action_name);
+
+  g_signal_connect_swapped (self->state->item_mngr,
+                            "items-changed",
+                            G_CALLBACK (photos_application_items_changed),
+                            self);
   g_signal_connect_swapped (self->state->item_mngr,
                             "load-finished",
                             G_CALLBACK (photos_application_load_changed),
@@ -2158,7 +2201,9 @@ photos_application_dispose (GObject *object)
   g_clear_object (&self->set_ss_action);
   g_clear_object (&self->share_action);
   g_clear_object (&self->sharpen_action);
+  g_clear_object (&self->zoom_begin_action);
   g_clear_object (&self->zoom_best_fit_action);
+  g_clear_object (&self->zoom_end_action);
   g_clear_object (&self->zoom_in_action);
   g_clear_object (&self->zoom_out_action);
   g_clear_object (&self->shr_pnt_mngr);
