@@ -202,7 +202,7 @@ photos_camera_cache_get_camera_async (PhotosCameraCache *self,
 {
   GApplication *app;
   GTask *task;
-  PhotosQuery *query;
+  PhotosQuery *query = NULL;
   PhotosSearchContextState *state;
   const gchar *camera;
 
@@ -228,14 +228,14 @@ photos_camera_cache_get_camera_async (PhotosCameraCache *self,
 
   query = photos_query_builder_equipment_query (state, id);
   photos_tracker_queue_select (self->queue,
-                               query->sparql,
+                               query,
                                cancellable,
                                photos_camera_cache_equipment_query_executed,
                                g_object_ref (task),
                                g_object_unref);
-  photos_query_free (query);
 
  out:
+  g_clear_object (&query);
   g_object_unref (task);
 }
 
