@@ -202,7 +202,7 @@ photos_create_collection_job_run (PhotosCreateCollectionJob *self,
 {
   GApplication *app;
   GTask *task;
-  PhotosQuery *query;
+  PhotosQuery *query = NULL;
   PhotosSearchContextState *state;
 
   task = g_task_new (self, cancellable, callback, user_data);
@@ -219,13 +219,13 @@ photos_create_collection_job_run (PhotosCreateCollectionJob *self,
 
   query = photos_query_builder_create_collection_query (state, self->name);
   photos_tracker_queue_update_blank (self->queue,
-                                     query->sparql,
+                                     query,
                                      cancellable,
                                      photos_create_collection_job_query_executed,
                                      g_object_ref (task),
                                      g_object_unref);
-  photos_query_free (query);
 
  out:
+  g_clear_object (&query);
   g_object_unref (task);
 }
