@@ -59,6 +59,7 @@ enum
   EXPORT_TIMEOUT = 10 /* s */
 };
 
+
 static void
 photos_export_notification_remove_timeout (PhotosExportNotification *self)
 {
@@ -104,79 +105,6 @@ photos_export_notification_close (PhotosExportNotification *self)
 }
 
 
-static void
-photos_export_notification_empty_trash_call_empty_trash (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GDBusConnection *connection = G_DBUS_CONNECTION (source_object);
-  GError *error = NULL;
-  GVariant *variant = NULL;
-
-  variant = g_dbus_connection_call_finish (connection, res, &error);
-  if (error != NULL)
-    {
-      if (g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN)
-          || g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_INTERFACE)
-          || g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD)
-          || g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_OBJECT))
-        {
-          g_warning ("Unable to call org.gnome.SettingsDaemon.Housekeeping:EmptyTrash: "
-                     "unknown interface/method/object/service");
-        }
-      else
-        {
-          g_warning ("Unable to call org.gnome.SettingsDaemon.Housekeeping:EmptyTrash: %s", error->message);
-        }
-
-      g_error_free (error);
-    }
-
-  g_clear_pointer (&variant, (GDestroyNotify) g_variant_unref);
-}
-
-
-static void
-photos_export_notification_empty_trash_call_empty_trash_pre_3_24 (GObject *source_object,
-                                                                  GAsyncResult *res,
-                                                                  gpointer user_data)
-{
-  GDBusConnection *connection = G_DBUS_CONNECTION (source_object);
-  GError *error = NULL;
-  GVariant *variant = NULL;
-
-  variant = g_dbus_connection_call_finish (connection, res, &error);
-  if (error != NULL)
-    {
-      if (g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN)
-          || g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_INTERFACE)
-          || g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD)
-          || g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_OBJECT))
-        {
-          g_dbus_connection_call (connection,
-                                  "org.gnome.SettingsDaemon.Housekeeping",
-                                  "/org/gnome/SettingsDaemon/Housekeeping",
-                                  "org.gnome.SettingsDaemon.Housekeeping",
-                                  "EmptyTrash",
-                                  NULL,
-                                  NULL,
-                                  G_DBUS_CALL_FLAGS_NONE,
-                                  -1,
-                                  NULL,
-                                  photos_export_notification_empty_trash_call_empty_trash,
-                                  NULL);
-        }
-      else
-        {
-          g_warning ("Unable to call org.gnome.SettingsDaemon.Housekeeping:EmptyTrash (GNOME < 3.24): %s",
-                     error->message);
-        }
-
-      g_error_free (error);
-    }
-
-  g_clear_pointer (&variant, (GDestroyNotify) g_variant_unref);
-}
-
-
 static void
 photos_export_notification_empty_trash_bus_get (GObject *source_object, GAsyncResult *result, gpointer user_data)
 {
@@ -202,7 +130,7 @@ photos_export_notification_empty_trash_bus_get (GObject *source_object, GAsyncRe
                           G_DBUS_CALL_FLAGS_NONE,
                           -1,
                           NULL,
-                          photos_export_notification_empty_trash_call_empty_trash_pre_3_24,
+                          NULL,
                           NULL);
 
  out:
