@@ -237,7 +237,7 @@ photos_fetch_collections_job_run (PhotosFetchCollectionsJob *self,
 {
   GApplication *app;
   GTask *task;
-  PhotosQuery *query;
+  PhotosQuery *query = NULL;
   PhotosSearchContextState *state;
 
   task = g_task_new (self, cancellable, callback, user_data);
@@ -254,13 +254,13 @@ photos_fetch_collections_job_run (PhotosFetchCollectionsJob *self,
 
   query = photos_query_builder_fetch_collections_query (state, self->urn);
   photos_tracker_queue_select (self->queue,
-                               query->sparql,
+                               query,
                                cancellable,
                                photos_fetch_collections_job_query_executed,
                                g_object_ref (task),
                                g_object_unref);
-  photos_query_free (query);
 
  out:
+  g_clear_object (&query);
   g_object_unref (task);
 }
