@@ -49,7 +49,6 @@ struct _PhotosImageView
   cairo_region_t *bbox_region;
   cairo_region_t *region;
   gboolean best_fit;
-  gboolean queued_best_fit_animation;
   gdouble height;
   gdouble width;
   gdouble x;
@@ -96,6 +95,87 @@ static const guint ZOOM_ANIMATION_DURATION = 250; /* ms */
 static void photos_image_view_computed (PhotosImageView *self, GeglRectangle *rect);
 
 
+static gboolean
+photos_image_view_has_allocation_and_extent (PhotosImageView *self)
+{
+  GeglRectangle bbox;
+  GtkAllocation allocation;
+  gboolean ret_val = FALSE;
+
+  if (self->buffer == NULL)
+    goto out;
+
+  gtk_widget_get_allocation (GTK_WIDGET (self), &allocation);
+  if (allocation.width <= 0 || allocation.height <= 0)
+    goto out;
+
+  bbox = *gegl_buffer_get_extent (self->buffer);
+  if (bbox.width < 0 || bbox.height < 0)
+    goto out;
+
+  ret_val = TRUE;
+
+ out:
+  return ret_val;
+}
+
+
+static void
+photos_image_view_calculate_best_fit_zoom (PhotosImageView *self, gdouble *out_zoom, gdouble *out_zoom_scaled)
+{
+  GeglRectangle bbox;
+  GtkAllocation allocation;
+  gdouble zoom;
+  gdouble zoom_scaled = 1.0;
+  gint scale_factor;
+  gint viewport_height_real;
+  gint viewport_width_real;
+
+  if (!photos_image_view_has_allocation_and_extent (self))
+    goto out;
+
+  gtk_widget_get_allocation (GTK_WIDGET (self), &allocation);
+  bbox = *gegl_buffer_get_extent (self->buffer);
+
+  scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
+  viewport_height_real = allocation.height * scale_factor;
+  viewport_width_real = allocation.width * scale_factor;
+
+  if (bbox.height > viewport_height_real || bbox.width > viewport_width_real)
+    {
+      gdouble height_ratio = bbox.height / (gdouble) viewport_height_real;
+      gdouble width_ratio = bbox.width / (gdouble) viewport_width_real;
+      gdouble max_ratio =  MAX (height_ratio, width_ratio);
+
+      zoom_scaled = 1.0 / max_ratio;
+    }
+
+ out:
+  zoom = zoom_scaled / (gdouble) scale_factor;
+
+  if (out_zoom != NULL)
+    *out_zoom = zoom;
+
+  if (out_zoom_scaled != NULL)
+    *out_zoom_scaled = zoom_scaled;
+}
+
+
+static gboolean
+photos_image_view_needs_zoom_animation (PhotosImageView *self)
+{
+  gboolean ret_val = FALSE;
+
+  if (!photos_image_view_has_allocation_and_extent (self))
+    goto out;
+
+  ret_val = TRUE;
+
+ out:
+  return ret_val;
+}
+
+
 static void
 photos_image_view_notify_zoom (GObject *object, GParamSpec *pspec, gpointer user_data)
 {
@@ -114,6 +194,38 @@ photos_image_view_notify_zoom (GObject *object, GParamSpec *pspec, gpointer user
 }
 
 
+static void
+photos_image_view_start_zoom_animation (PhotosImageView *self)
+{
+  GdkFrameClock *frame_clock;
+  PhotosImageViewHelper *helper;
+
+  g_return_if_fail (photos_image_view_needs_zoom_animation (self));
+  g_return_if_fail (self->zoom > 0.0);
+  g_return_if_fail (self->zoom_animation == NULL);
+  g_return_if_fail (self->zoom_visible > 0.0);
+
+  helper = photos_image_view_helper_new ();
+  photos_image_view_helper_set_zoom (helper, self->zoom_visible);
+  g_signal_connect (helper, "notify::zoom", G_CALLBACK (photos_image_view_notify_zoom), self);
+
+  frame_clock = gtk_widget_get_frame_clock (GTK_WIDGET (self));
+
+  self->zoom_animation = egg_object_animate_full (g_object_ref (helper),
+                                                  ZOOM_ANIMATION_MODE,
+                                                  ZOOM_ANIMATION_DURATION,
+                                                  frame_clock,
+                                                  g_object_unref,
+                                                  helper,
+                                                  "zoom",
+                                                  self->zoom,
+                                                  NULL);
+  g_object_add_weak_pointer (G_OBJECT (self->zoom_animation), (gpointer *) &self->zoom_animation);
+
+  g_object_unref (helper);
+}
+
+
 static void
 photos_image_view_update_buffer (PhotosImageView *self)
 {
@@ -268,72 +380,46 @@ photos_image_view_update (PhotosImageView *self)
   if (self->node == NULL)
     goto out;
 
-  g_assert_true (GEGL_IS_BUFFER (self->buffer));
+  if (!photos_image_view_has_allocation_and_extent (self))
+    goto out;
+
+  if (self->best_fit)
+    {
+      gdouble zoom;
+      gdouble zoom_scaled;
+
+      photos_image_view_calculate_best_fit_zoom (self, &zoom, &zoom_scaled);
+
+      if (!photos_utils_equal_double (self->zoom, zoom))
+        {
+          self->zoom = zoom;
+          g_object_notify (G_OBJECT (self), "zoom");
+
+          if (self->zoom_animation != NULL)
+            {
+              egg_animation_stop (self->zoom_animation);
+              photos_image_view_start_zoom_animation (self);
+              goto out;
+            }
+
+          self->zoom_visible = self->zoom;
+          self->zoom_visible_scaled = zoom_scaled;
+        }
+    }
 
   gtk_widget_get_allocation (GTK_WIDGET (self), &allocation);
-
-  if (allocation.width <= 0 || allocation.height <= 0)
-    goto out;
-
   bbox = *gegl_buffer_get_extent (self->buffer);
-  if (bbox.width < 0 || bbox.height < 0)
-    goto out;
-
   scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
+
   viewport_height_real = allocation.height * scale_factor;
   viewport_width_real = allocation.width * scale_factor;
 
   if (self->best_fit && self->zoom_animation == NULL)
     {
-      gdouble zoom_scaled = 1.0;
-
-      if (bbox.height > viewport_height_real || bbox.width > viewport_width_real)
-        {
-          gdouble height_ratio = bbox.height / (gdouble) viewport_height_real;
-          gdouble width_ratio = bbox.width / (gdouble) viewport_width_real;
-          gdouble max_ratio =  MAX (height_ratio, width_ratio);
-
-          zoom_scaled = 1.0 / max_ratio;
-        }
-
-      bbox_zoomed.width = (gint) (zoom_scaled * bbox.width + 0.5);
-      bbox_zoomed.height = (gint) (zoom_scaled * bbox.height + 0.5);
-      bbox_zoomed.x = (gint) (zoom_scaled * bbox.x + 0.5);
-      bbox_zoomed.y = (gint) (zoom_scaled * bbox.y + 0.5);
-
-      self->zoom = zoom_scaled / (gdouble) scale_factor;
-      g_object_notify (G_OBJECT (self), "zoom");
-
-      if (self->zoom_visible > 0.0 && self->queued_best_fit_animation)
-        {
-          GdkFrameClock *frame_clock;
-          PhotosImageViewHelper *helper;
-
-          helper = photos_image_view_helper_new ();
-          photos_image_view_helper_set_zoom (helper, self->zoom_visible);
-          g_signal_connect (helper, "notify::zoom", G_CALLBACK (photos_image_view_notify_zoom), self);
-
-          frame_clock = gtk_widget_get_frame_clock (GTK_WIDGET (self));
-
-          self->zoom_animation = egg_object_animate_full (g_object_ref (helper),
-                                                          ZOOM_ANIMATION_MODE,
-                                                          ZOOM_ANIMATION_DURATION,
-                                                          frame_clock,
-                                                          g_object_unref,
-                                                          helper,
-                                                          "zoom",
-                                                          self->zoom,
-                                                          NULL);
-          g_object_add_weak_pointer (G_OBJECT (self->zoom_animation), (gpointer *) &self->zoom_animation);
-
-          g_object_unref (helper);
-          goto out;
-        }
-      else
-        {
-          self->zoom_visible = self->zoom;
-          self->zoom_visible_scaled = zoom_scaled;
-        }
+      bbox_zoomed.width = (gint) (self->zoom_visible_scaled * bbox.width + 0.5);
+      bbox_zoomed.height = (gint) (self->zoom_visible_scaled * bbox.height + 0.5);
+      bbox_zoomed.x = (gint) (self->zoom_visible_scaled * bbox.x + 0.5);
+      bbox_zoomed.y = (gint) (self->zoom_visible_scaled * bbox.y + 0.5);
 
       self->x_scaled = (bbox_zoomed.width - viewport_width_real) / 2.0;
       self->y_scaled = (bbox_zoomed.height - viewport_height_real) / 2.0;
@@ -546,8 +632,6 @@ photos_image_view_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
   scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
   self->allocation_scaled_old.height = allocation->height * scale_factor;
   self->allocation_scaled_old.width = allocation->width * scale_factor;
-
-  self->queued_best_fit_animation = FALSE;
 }
 
 
@@ -723,7 +807,7 @@ photos_image_view_set_property (GObject *object, guint prop_id, const GValue *va
         gboolean best_fit;
 
         best_fit = g_value_get_boolean (value);
-        photos_image_view_set_best_fit (self, best_fit);
+        photos_image_view_set_best_fit (self, best_fit, TRUE);
         break;
       }
 
@@ -777,7 +861,7 @@ photos_image_view_set_property (GObject *object, guint prop_id, const GValue *va
         gdouble zoom;
 
         zoom = g_value_get_double (value);
-        photos_image_view_set_zoom (self, zoom);
+        photos_image_view_set_zoom (self, zoom, TRUE);
         break;
       }
 
@@ -805,6 +889,9 @@ photos_image_view_init (PhotosImageView *self)
   gtk_style_context_add_class (context, "content-view");
 
   self->best_fit = TRUE;
+  self->zoom = 1.0;
+  self->zoom_visible = 1.0;
+  self->zoom_visible_scaled = 1.0;
 }
 
 
@@ -958,7 +1045,7 @@ photos_image_view_get_zoom (PhotosImageView *self)
 
 
 void
-photos_image_view_set_best_fit (PhotosImageView *self, gboolean best_fit)
+photos_image_view_set_best_fit (PhotosImageView *self, gboolean best_fit, gboolean enable_animation)
 {
   g_return_if_fail (PHOTOS_IS_IMAGE_VIEW (self));
 
@@ -969,12 +1056,24 @@ photos_image_view_set_best_fit (PhotosImageView *self, gboolean best_fit)
 
   if (self->best_fit)
     {
+      gdouble zoom_scaled;
+
       if (self->zoom_animation != NULL)
         egg_animation_stop (self->zoom_animation);
 
-      self->zoom = 0.0;
-      self->queued_best_fit_animation = TRUE;
-      gtk_widget_queue_resize (GTK_WIDGET (self));
+      photos_image_view_calculate_best_fit_zoom (self, &self->zoom, &zoom_scaled);
+
+      if (enable_animation && photos_image_view_needs_zoom_animation (self))
+        {
+          photos_image_view_start_zoom_animation (self);
+        }
+      else
+        {
+          self->zoom_visible = self->zoom;
+          self->zoom_visible_scaled = zoom_scaled;
+          gtk_widget_queue_resize (GTK_WIDGET (self));
+        }
+
       g_object_notify (G_OBJECT (self), "zoom");
     }
 
@@ -985,12 +1084,17 @@ photos_image_view_set_best_fit (PhotosImageView *self, gboolean best_fit)
 void
 photos_image_view_set_node (PhotosImageView *self, GeglNode *node)
 {
+  gdouble zoom_scaled;
+
   g_return_if_fail (PHOTOS_IS_IMAGE_VIEW (self));
   g_return_if_fail (node == NULL || GEGL_IS_NODE (node));
 
   if (self->node == node)
     return;
 
+  if (self->zoom_animation != NULL)
+    egg_animation_stop (self->zoom_animation);
+
   if (self->node != NULL)
     {
       g_signal_handlers_disconnect_by_func (self->node, photos_image_view_computed, self);
@@ -1006,9 +1110,9 @@ photos_image_view_set_node (PhotosImageView *self, GeglNode *node)
   self->x_scaled = 0.0;
   self->y = 0.0;
   self->y_scaled = 0.0;
-  self->zoom = 0.0;
-  self->zoom_visible = 0.0;
-  self->zoom_visible_scaled = 0.0;
+  self->zoom = 1.0;
+  self->zoom_visible = 1.0;
+  self->zoom_visible_scaled = 1.0;
   g_clear_object (&self->buffer);
   g_clear_object (&self->node);
   g_clear_pointer (&self->bbox_region, (GDestroyNotify) cairo_region_destroy);
@@ -1029,6 +1133,10 @@ photos_image_view_set_node (PhotosImageView *self, GeglNode *node)
                                G_CONNECT_SWAPPED);
     }
 
+  photos_image_view_calculate_best_fit_zoom (self, &self->zoom, &zoom_scaled);
+  self->zoom_visible = self->zoom;
+  self->zoom_visible_scaled = zoom_scaled;
+
   gtk_widget_queue_resize (GTK_WIDGET (self));
 
   g_object_notify (G_OBJECT (self), "best-fit");
@@ -1038,9 +1146,10 @@ photos_image_view_set_node (PhotosImageView *self, GeglNode *node)
 
 
 void
-photos_image_view_set_zoom (PhotosImageView *self, gdouble zoom)
+photos_image_view_set_zoom (PhotosImageView *self, gdouble zoom, gboolean enable_animation)
 {
   g_return_if_fail (PHOTOS_IS_IMAGE_VIEW (self));
+  g_return_if_fail (zoom > 0.0);
 
   if (photos_utils_equal_double (self->zoom, zoom))
     return;
@@ -1051,29 +1160,9 @@ photos_image_view_set_zoom (PhotosImageView *self, gdouble zoom)
   self->best_fit = FALSE;
   self->zoom = zoom;
 
-  if (self->zoom_visible > 0.0)
+  if (enable_animation && photos_image_view_needs_zoom_animation (self))
     {
-      GdkFrameClock *frame_clock;
-      PhotosImageViewHelper *helper;
-
-      helper = photos_image_view_helper_new ();
-      photos_image_view_helper_set_zoom (helper, self->zoom_visible);
-      g_signal_connect (helper, "notify::zoom", G_CALLBACK (photos_image_view_notify_zoom), self);
-
-      frame_clock = gtk_widget_get_frame_clock (GTK_WIDGET (self));
-
-      self->zoom_animation = egg_object_animate_full (g_object_ref (helper),
-                                                      ZOOM_ANIMATION_MODE,
-                                                      ZOOM_ANIMATION_DURATION,
-                                                      frame_clock,
-                                                      g_object_unref,
-                                                      helper,
-                                                      "zoom",
-                                                      self->zoom,
-                                                      NULL);
-      g_object_add_weak_pointer (G_OBJECT (self->zoom_animation), (gpointer *) &self->zoom_animation);
-
-      g_object_unref (helper);
+      photos_image_view_start_zoom_animation (self);
     }
   else
     {
