@@ -48,10 +48,8 @@ struct _PhotosMainToolbar
   GtkBox parent_instance;
   GAction *search;
   GSimpleAction *gear_menu;
-  GtkWidget *coll_back_button;
   GtkWidget *favorite_button;
   GtkWidget *header_bar;
-  GtkWidget *overlay;
   GtkWidget *remote_display_button;
   GtkWidget *searchbar;
   GtkWidget *selection_menu;
@@ -61,12 +59,6 @@ struct _PhotosMainToolbar
   PhotosSelectionController *sel_cntrlr;
 };
 
-enum
-{
-  PROP_0,
-  PROP_OVERLAY
-};
-
 
 G_DEFINE_TYPE (PhotosMainToolbar, photos_main_toolbar, GTK_TYPE_BOX);
 
@@ -117,7 +109,8 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
 
   if (selection_mode)
     {
-      g_return_if_fail (window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+      g_return_if_fail (window_mode == PHOTOS_WINDOW_MODE_COLLECTION_VIEW
+                        || window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS
                         || window_mode == PHOTOS_WINDOW_MODE_FAVORITES
                         || window_mode == PHOTOS_WINDOW_MODE_OVERVIEW
                         || window_mode == PHOTOS_WINDOW_MODE_SEARCH);
@@ -127,20 +120,21 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
     g_return_if_fail (!selection_mode);
 
   active_collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (self->item_mngr));
-  if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW
-      || window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS
-      || window_mode == PHOTOS_WINDOW_MODE_FAVORITES
-      || window_mode == PHOTOS_WINDOW_MODE_SEARCH)
+
+  if (window_mode == PHOTOS_WINDOW_MODE_COLLECTION_VIEW)
     {
-      if (!selection_mode)
-        {
-          if (active_collection != NULL)
-            primary = g_strdup (photos_base_item_get_name (active_collection));
-        }
+      if (selection_mode)
+        primary = photos_main_toolbar_create_selection_mode_label (self, active_collection);
       else
-        {
-          primary = photos_main_toolbar_create_selection_mode_label (self, active_collection);
-        }
+        primary = g_strdup (photos_base_item_get_name (active_collection));
+    }
+  else if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW
+           || window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+           || window_mode == PHOTOS_WINDOW_MODE_FAVORITES
+           || window_mode == PHOTOS_WINDOW_MODE_SEARCH)
+    {
+      if (selection_mode)
+        primary = photos_main_toolbar_create_selection_mode_label (self, NULL);
     }
   else if (window_mode == PHOTOS_WINDOW_MODE_EDIT || window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
     {
@@ -169,6 +163,13 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
 }
 
 
+static void
+photos_main_toolbar_back_button_clicked (PhotosMainToolbar *self)
+{
+  photos_mode_controller_go_back (self->mode_cntrlr);
+}
+
+
 static GtkWidget *
 photos_main_toolbar_add_back_button (PhotosMainToolbar *self)
 {
@@ -177,6 +178,7 @@ photos_main_toolbar_add_back_button (PhotosMainToolbar *self)
   back_button = gtk_button_new_from_icon_name (PHOTOS_ICON_GO_PREVIOUS_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
   gtk_widget_set_tooltip_text (back_button, _("Back"));
   gtk_header_bar_pack_start (GTK_HEADER_BAR (self->header_bar), back_button);
+  g_signal_connect_swapped (back_button, "clicked", G_CALLBACK (photos_main_toolbar_back_button_clicked), self);
 
   return back_button;
 }
@@ -236,44 +238,6 @@ photos_main_toolbar_update_remote_display_button (PhotosMainToolbar *self)
 }
 
 
-static void
-photos_main_toolbar_coll_back_button_clicked (PhotosMainToolbar *self)
-{
-  photos_item_manager_activate_previous_collection (PHOTOS_ITEM_MANAGER (self->item_mngr));
-}
-
-
-static void
-photos_main_toolbar_col_active_changed (PhotosMainToolbar *self, PhotosBaseItem *collection)
-{
-  PhotosHeaderBarMode mode;
-
-  if (collection != NULL)
-    {
-      mode = PHOTOS_HEADER_BAR_MODE_STANDALONE;
-      if (self->coll_back_button == NULL)
-        {
-          self->coll_back_button = photos_main_toolbar_add_back_button (self);
-          gtk_widget_show (self->coll_back_button);
-
-          g_signal_connect_swapped (self->coll_back_button,
-                                    "clicked",
-                                    G_CALLBACK (photos_main_toolbar_coll_back_button_clicked),
-                                    self);
-        }
-    }
-  else
-    {
-      mode = PHOTOS_HEADER_BAR_MODE_NORMAL;
-      g_clear_pointer (&self->coll_back_button, (GDestroyNotify) gtk_widget_destroy);
-    }
-
-  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (self->header_bar), mode);
-  photos_main_toolbar_update_remote_display_button (self);
-  photos_main_toolbar_set_toolbar_title (self);
-}
-
-
 static void
 photos_main_toolbar_item_active_changed (PhotosMainToolbar *self, GObject *object)
 {
@@ -321,34 +285,17 @@ photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
   gtk_actionable_set_action_name (GTK_ACTIONABLE (selection_button), "app.selection-mode");
   gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header_bar), selection_button);
 
-  g_signal_connect_object (self->item_mngr,
-                           "active-collection-changed",
-                           G_CALLBACK (photos_main_toolbar_col_active_changed),
-                           self,
-                           G_CONNECT_SWAPPED);
-
   return selection_button;
 }
 
 
-static void
-photos_main_toolbar_back_button_clicked (PhotosMainToolbar *self)
-{
-  photos_mode_controller_go_back (self->mode_cntrlr);
-}
-
-
 static void
 photos_main_toolbar_clear_state_data (PhotosMainToolbar *self)
 {
-  g_clear_pointer (&self->coll_back_button, (GDestroyNotify) gtk_widget_destroy);
   g_clear_pointer (&self->remote_display_button, (GDestroyNotify) gtk_widget_destroy);
 
   if (self->item_mngr != NULL)
-    {
-      g_signal_handlers_disconnect_by_func (self->item_mngr, photos_main_toolbar_col_active_changed, self);
-      g_signal_handlers_disconnect_by_func (self->item_mngr, photos_main_toolbar_item_active_changed, self);
-    }
+    g_signal_handlers_disconnect_by_func (self->item_mngr, photos_main_toolbar_item_active_changed, self);
 
   if (self->sel_cntrlr != NULL)
     g_signal_handlers_disconnect_by_func (self->sel_cntrlr, photos_main_toolbar_set_toolbar_title, self);
@@ -437,18 +384,24 @@ photos_main_toolbar_favorite_button_update (PhotosMainToolbar *self, gboolean fa
 }
 
 
+static void
+photos_main_toolbar_populate_for_collection_view (PhotosMainToolbar *self)
+{
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), TRUE);
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (self->header_bar), PHOTOS_HEADER_BAR_MODE_STANDALONE);
+  photos_main_toolbar_add_back_button (self);
+  photos_main_toolbar_add_selection_button (self);
+  photos_main_toolbar_add_search_button (self);
+}
+
+
 static void
 photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
 {
-  PhotosBaseItem *collection;
-
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (self->header_bar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
   photos_main_toolbar_add_search_button (self);
-
-  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (self->item_mngr));
-  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
@@ -476,38 +429,27 @@ photos_main_toolbar_populate_for_edit (PhotosMainToolbar *self)
 static void
 photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
 {
-  PhotosBaseItem *collection;
-
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (self->header_bar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
   photos_main_toolbar_add_search_button (self);
-
-  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (self->item_mngr));
-  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
 static void
 photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
 {
-  PhotosBaseItem *collection;
-
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (self->header_bar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
   photos_main_toolbar_add_search_button (self);
-
-  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (self->item_mngr));
-  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
 static void
 photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
 {
-  GMenu *preview_menu = NULL;
-  GtkWidget *back_button;
+  GMenu *preview_menu;
   GtkWidget *edit_button;
   GtkWidget *image;
   GtkWidget *menu_button;
@@ -521,8 +463,7 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (self->header_bar), PHOTOS_HEADER_BAR_MODE_STANDALONE);
 
-  back_button = photos_main_toolbar_add_back_button (self);
-  g_signal_connect_swapped (back_button, "clicked", G_CALLBACK (photos_main_toolbar_back_button_clicked), self);
+  photos_main_toolbar_add_back_button (self);
 
   preview_menu = photos_main_toolbar_create_preview_menu (self);
   image = gtk_image_new_from_icon_name (PHOTOS_ICON_SYSTEM_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
@@ -565,23 +506,16 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
                            G_CALLBACK (photos_main_toolbar_item_active_changed),
                            self,
                            G_CONNECT_SWAPPED);
-
-  g_object_unref (preview_menu);
 }
 
 
 static void
 photos_main_toolbar_populate_for_search (PhotosMainToolbar *self)
 {
-  PhotosBaseItem *collection;
-
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (self->header_bar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
   photos_main_toolbar_add_search_button (self);
-
-  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (self->item_mngr));
-  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
@@ -633,24 +567,6 @@ photos_main_toolbar_dispose (GObject *object)
 }
 
 
-static void
-photos_main_toolbar_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
-{
-  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (object);
-
-  switch (prop_id)
-    {
-    case PROP_OVERLAY:
-      self->overlay = GTK_WIDGET (g_value_dup_object (value));
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-
 static void
 photos_main_toolbar_share_changed_cb (PhotosMainToolbar          *self,
                                       PhotosDlnaRenderer         *renderer,
@@ -738,15 +654,6 @@ photos_main_toolbar_class_init (PhotosMainToolbarClass *class)
 
   object_class->constructed = photos_main_toolbar_constructed;
   object_class->dispose = photos_main_toolbar_dispose;
-  object_class->set_property = photos_main_toolbar_set_property;
-
-  g_object_class_install_property (object_class,
-                                   PROP_OVERLAY,
-                                   g_param_spec_object ("overlay",
-                                                        "GtkOverlay object",
-                                                        "The stack overlay widget",
-                                                        GTK_TYPE_OVERLAY,
-                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Photos/main-toolbar.ui");
   gtk_widget_class_bind_template_child (widget_class, PhotosMainToolbar, header_bar);
@@ -755,9 +662,9 @@ photos_main_toolbar_class_init (PhotosMainToolbarClass *class)
 
 
 GtkWidget *
-photos_main_toolbar_new (GtkOverlay *overlay)
+photos_main_toolbar_new (void)
 {
-  return g_object_new (PHOTOS_TYPE_MAIN_TOOLBAR, "overlay", overlay, NULL);
+  return g_object_new (PHOTOS_TYPE_MAIN_TOOLBAR, NULL);
 }
 
 
@@ -808,6 +715,10 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
 
       switch (window_mode)
         {
+        case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
+          photos_main_toolbar_populate_for_collection_view (self);
+          break;
+
         case PHOTOS_WINDOW_MODE_COLLECTIONS:
           photos_main_toolbar_populate_for_collections (self);
           break;
