@@ -41,9 +41,15 @@ PhotosPreviewNavButtons    *photos_preview_nav_buttons_new                (Photo
 
 void                        photos_preview_nav_buttons_hide               (PhotosPreviewNavButtons *self);
 
+void                        photos_preview_nav_buttons_set_auto_hide      (PhotosPreviewNavButtons *self,
+                                                                           gboolean auto_hide);
+
 void                        photos_preview_nav_buttons_set_mode           (PhotosPreviewNavButtons *self,
                                                                            PhotosWindowMode old_mode);
 
+void                        photos_preview_nav_buttons_set_show_navigation (PhotosPreviewNavButtons *self,
+                                                                            gboolean show_navigation);
+
 void                        photos_preview_nav_buttons_show               (PhotosPreviewNavButtons *self);
 
 G_END_DECLS
