@@ -34,6 +34,7 @@
 #include "photos-application.h"
 #include "photos-base-item.h"
 #include "photos-done-notification.h"
+#include "photos-gesture-zoom.h"
 #include "photos-image-view.h"
 #include "photos-edit-palette.h"
 #include "photos-operation-insta-common.h"
@@ -47,7 +48,9 @@
 struct _PhotosPreviewView
 {
   GtkBin parent_instance;
+  GAction *zoom_begin_action;
   GAction *zoom_best_fit_action;
+  GAction *zoom_end_action;
   GAction *zoom_in_action;
   GAction *zoom_out_action;
   GCancellable *cancellable;
@@ -57,20 +60,17 @@ struct _PhotosPreviewView
   GtkWidget *revealer;
   GtkWidget *stack;
   PhotosBaseManager *item_mngr;
+  PhotosGestureZoom *gesture_zoom;
   PhotosModeController *mode_cntrlr;
   PhotosPreviewNavButtons *nav_buttons;
   PhotosTool *current_tool;
   gboolean grabbed;
   gdouble event_x_last;
   gdouble event_y_last;
+  gdouble zoom_begin;
   gdouble zoom_best_fit;
 };
 
-struct _PhotosPreviewViewClass
-{
-  GtkBinClass parent_class;
-};
-
 enum
 {
   PROP_0,
@@ -81,6 +81,7 @@ enum
 G_DEFINE_TYPE (PhotosPreviewView, photos_preview_view, GTK_TYPE_BIN);
 
 
+static const gdouble ZOOM_BEST_FIT_FACTOR_TOUCH = 0.6;
 static const gdouble ZOOM_FACTOR_1 = 2.8561;
 static const gdouble ZOOM_FACTOR_2 = 1.69;
 static const gdouble ZOOM_FACTOR_3 = 1.3;
@@ -251,6 +252,79 @@ photos_preview_view_draw_overlay (PhotosPreviewView *self, cairo_t *cr, GdkRecta
 }
 
 
+static void
+photos_preview_view_gesture_zoom_begin (PhotosPreviewView *self)
+{
+  GVariant *parameter;
+  GtkGesture *gesture_zoom;
+
+  gesture_zoom = photos_gesture_zoom_get_gesture (self->gesture_zoom);
+  gtk_gesture_set_state (gesture_zoom, GTK_EVENT_SEQUENCE_CLAIMED);
+
+  parameter = photos_utils_create_zoom_target_value (0.0, PHOTOS_ZOOM_EVENT_TOUCH);
+  g_assert_true (g_variant_is_floating (parameter));
+
+  g_action_activate (self->zoom_begin_action, parameter);
+}
+
+
+static void
+photos_preview_view_gesture_zoom_direction_changed (PhotosPreviewView *self)
+{
+  GVariant *parameter;
+
+  parameter = photos_utils_create_zoom_target_value (0.0, PHOTOS_ZOOM_EVENT_TOUCH);
+  g_assert_true (g_variant_is_floating (parameter));
+
+  g_action_activate (self->zoom_end_action, parameter);
+
+  parameter = photos_utils_create_zoom_target_value (0.0, PHOTOS_ZOOM_EVENT_TOUCH);
+  g_assert_true (g_variant_is_floating (parameter));
+
+  g_action_activate (self->zoom_begin_action, parameter);
+}
+
+
+static void
+photos_preview_view_gesture_zoom_end (PhotosPreviewView *self)
+{
+  GVariant *parameter;
+
+  parameter = photos_utils_create_zoom_target_value (0.0, PHOTOS_ZOOM_EVENT_TOUCH);
+  g_assert_true (g_variant_is_floating (parameter));
+
+  g_action_activate (self->zoom_end_action, parameter);
+}
+
+
+static void
+photos_preview_view_gesture_zoom_scale_changed (PhotosPreviewView *self,
+                                                gdouble scale,
+                                                PhotosGestureZoomDirection direction)
+{
+  GVariant *parameter;
+
+  parameter = photos_utils_create_zoom_target_value (scale, PHOTOS_ZOOM_EVENT_TOUCH);
+  g_assert_true (g_variant_is_floating (parameter));
+
+  switch (direction)
+    {
+    case PHOTOS_GESTURE_ZOOM_DIRECTION_DECREASING:
+      g_action_activate (self->zoom_out_action, parameter);
+      break;
+
+    case PHOTOS_GESTURE_ZOOM_DIRECTION_INCREASING:
+      g_action_activate (self->zoom_in_action, parameter);
+      break;
+
+    case PHOTOS_GESTURE_ZOOM_DIRECTION_NONE:
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+}
+
+
 static GtkWidget *
 photos_preview_view_get_invisible_child (PhotosPreviewView *self)
 {
@@ -330,7 +404,7 @@ photos_preview_view_update_zoom_best_fit (PhotosPreviewView *self, PhotosImageVi
   g_return_if_fail (view == PHOTOS_IMAGE_VIEW (current_view));
 
   zoom = photos_image_view_get_zoom (view);
-  if (!photos_image_view_get_best_fit (view) || zoom == 0.0)
+  if (!photos_image_view_get_best_fit (view))
     return;
 
   self->zoom_best_fit = zoom;
@@ -415,7 +489,7 @@ photos_preview_view_scroll_event (GtkWidget *widget, GdkEvent *event, gpointer u
     goto out;
 
   delta_abs = fabs (event->scroll.delta_y);
-  parameter = g_variant_new_double (delta_abs);
+  parameter = photos_utils_create_zoom_target_value (delta_abs, PHOTOS_ZOOM_EVENT_SCROLL);
   parameter = g_variant_ref_sink (parameter);
 
   if (event->scroll.delta_y < 0.0)
@@ -807,6 +881,7 @@ photos_preview_view_window_mode_changed (PhotosPreviewView *self, PhotosWindowMo
 {
   switch (mode)
     {
+    case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
     case PHOTOS_WINDOW_MODE_COLLECTIONS:
     case PHOTOS_WINDOW_MODE_FAVORITES:
     case PHOTOS_WINDOW_MODE_OVERVIEW:
@@ -839,6 +914,29 @@ photos_preview_view_window_mode_changed (PhotosPreviewView *self, PhotosWindowMo
 }
 
 
+static void
+photos_preview_view_zoom_begin (PhotosPreviewView *self, GVariant *parameter)
+{
+  GtkWidget *view;
+  GtkWidget *view_container;
+  PhotosZoomEvent event;
+
+  event = photos_utils_get_zoom_event (parameter);
+  g_return_if_fail (event == PHOTOS_ZOOM_EVENT_TOUCH);
+
+  view_container = gtk_stack_get_visible_child (GTK_STACK (self->stack));
+  view = photos_preview_view_get_view_from_view_container (view_container);
+  self->zoom_begin = photos_image_view_get_zoom (PHOTOS_IMAGE_VIEW (view));
+
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_begin_action), FALSE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_best_fit_action), TRUE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_end_action), TRUE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_out_action), TRUE);
+  photos_preview_nav_buttons_set_auto_hide (self->nav_buttons, FALSE);
+  photos_preview_nav_buttons_set_show_navigation (self->nav_buttons, FALSE);
+}
+
+
 static void
 photos_preview_view_zoom_best_fit (PhotosPreviewView *self)
 {
@@ -847,7 +945,7 @@ photos_preview_view_zoom_best_fit (PhotosPreviewView *self)
 
   view_container = gtk_stack_get_visible_child (GTK_STACK (self->stack));
   view = photos_preview_view_get_view_from_view_container (view_container);
-  photos_image_view_set_best_fit (PHOTOS_IMAGE_VIEW (view), TRUE);
+  photos_image_view_set_best_fit (PHOTOS_IMAGE_VIEW (view), TRUE, TRUE);
   g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_best_fit_action), FALSE);
   g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_out_action), FALSE);
   photos_preview_nav_buttons_set_auto_hide (self->nav_buttons, TRUE);
@@ -856,39 +954,135 @@ photos_preview_view_zoom_best_fit (PhotosPreviewView *self)
 
 
 static void
-photos_preview_view_zoom_in (PhotosPreviewView *self, GVariant *parameter)
+photos_preview_view_zoom_end (PhotosPreviewView *self, GVariant *parameter)
+{
+  GtkWidget *view;
+  GtkWidget *view_container;
+  PhotosZoomEvent event;
+  gdouble zoom;
+
+  event = photos_utils_get_zoom_event (parameter);
+  g_return_if_fail (event == PHOTOS_ZOOM_EVENT_TOUCH);
+
+  view_container = gtk_stack_get_visible_child (GTK_STACK (self->stack));
+  view = photos_preview_view_get_view_from_view_container (view_container);
+  zoom = photos_image_view_get_zoom (PHOTOS_IMAGE_VIEW (view));
+
+  if (zoom < self->zoom_best_fit || photos_utils_equal_double (self->zoom_best_fit, zoom))
+    {
+      photos_image_view_set_best_fit (PHOTOS_IMAGE_VIEW (view), TRUE, TRUE);
+      g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_best_fit_action), FALSE);
+      g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_out_action), FALSE);
+    }
+
+  self->zoom_begin = 0.0;
+
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_begin_action), TRUE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_end_action), FALSE);
+  photos_preview_nav_buttons_set_auto_hide (self->nav_buttons, TRUE);
+  photos_preview_nav_buttons_set_show_navigation (self->nav_buttons, TRUE);
+}
+
+
+static gdouble
+photos_preview_view_calculate_zoom_for_non_touch (PhotosPreviewView *self,
+                                                  gdouble delta,
+                                                  PhotosZoomEvent event,
+                                                  gboolean zoom_in)
 {
   GtkWidget *view;
   GtkWidget *view_container;
   gboolean best_fit;
-  gdouble delta;
-  gdouble delta_abs;
-  gdouble zoom;
+  gdouble zoom = 0.0;
   gdouble zoom_factor;
   gdouble zoom_factor_for_delta;
 
-  g_return_if_fail (self->zoom_best_fit > 0.0);
-
-  delta = g_variant_get_double (parameter);
-  g_return_if_fail (photos_utils_equal_double (delta, -1.0) || delta >= 0.0);
+  g_return_val_if_fail (event != PHOTOS_ZOOM_EVENT_NONE, 0.0);
+  g_return_val_if_fail (event != PHOTOS_ZOOM_EVENT_TOUCH, 0.0);
+  g_return_val_if_fail ((event == PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR && photos_utils_equal_double (delta, 1.0))
+                        || (event == PHOTOS_ZOOM_EVENT_MOUSE_CLICK && photos_utils_equal_double (delta, 1.0))
+                        || (event == PHOTOS_ZOOM_EVENT_SCROLL && delta >= 0.0),
+                        0.0);
 
   view_container = gtk_stack_get_visible_child (GTK_STACK (self->stack));
   view = photos_preview_view_get_view_from_view_container (view_container);
 
   best_fit = photos_image_view_get_best_fit (PHOTOS_IMAGE_VIEW (view));
 
-  if (delta >= 0.0)
-    zoom_factor = best_fit ? ZOOM_FACTOR_2 : ZOOM_FACTOR_3;
-  else
-    zoom_factor = best_fit ? ZOOM_FACTOR_1 : ZOOM_FACTOR_2;
+  switch (event)
+    {
+    case PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR:
+    case PHOTOS_ZOOM_EVENT_MOUSE_CLICK:
+      zoom_factor = best_fit && zoom_in ? ZOOM_FACTOR_1 : ZOOM_FACTOR_2;
+      break;
 
-  delta_abs = fabs (delta);
-  zoom_factor_for_delta = pow (zoom_factor, delta_abs);
+    case PHOTOS_ZOOM_EVENT_SCROLL:
+      zoom_factor = best_fit && zoom_in ? ZOOM_FACTOR_2 : ZOOM_FACTOR_3;
+      break;
 
+    case PHOTOS_ZOOM_EVENT_NONE:
+    case PHOTOS_ZOOM_EVENT_TOUCH:
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+
+  zoom_factor_for_delta = pow (zoom_factor, delta);
   zoom = photos_image_view_get_zoom (PHOTOS_IMAGE_VIEW (view));
-  zoom *= zoom_factor_for_delta;
 
-  photos_image_view_set_zoom (PHOTOS_IMAGE_VIEW (view), zoom);
+  if (zoom_in)
+    zoom *= zoom_factor_for_delta;
+  else
+    zoom /= zoom_factor_for_delta;
+
+  return zoom;
+}
+
+
+static void
+photos_preview_view_zoom_in (PhotosPreviewView *self, GVariant *parameter)
+{
+  GtkWidget *view;
+  GtkWidget *view_container;
+  PhotosZoomEvent event;
+  gboolean enable_animation;
+  gdouble delta;
+  gdouble zoom;
+
+  g_return_if_fail (self->zoom_best_fit > 0.0);
+
+  event = photos_utils_get_zoom_event (parameter);
+  g_return_if_fail (event != PHOTOS_ZOOM_EVENT_NONE);
+
+  delta = photos_utils_get_zoom_delta (parameter);
+  g_return_if_fail ((event == PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR && photos_utils_equal_double (delta, 1.0))
+                    || (event == PHOTOS_ZOOM_EVENT_MOUSE_CLICK && photos_utils_equal_double (delta, 1.0))
+                    || (event == PHOTOS_ZOOM_EVENT_SCROLL && delta >= 0.0)
+                    || (event == PHOTOS_ZOOM_EVENT_TOUCH && delta >= 0.0 && self->zoom_begin > 0.0));
+
+  switch (event)
+    {
+    case PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR:
+    case PHOTOS_ZOOM_EVENT_MOUSE_CLICK:
+    case PHOTOS_ZOOM_EVENT_SCROLL:
+      enable_animation = TRUE;
+      zoom = photos_preview_view_calculate_zoom_for_non_touch (self, delta, event, TRUE);
+      break;
+
+    case PHOTOS_ZOOM_EVENT_TOUCH:
+      enable_animation = FALSE;
+      zoom = self->zoom_begin * delta;
+      break;
+
+    case PHOTOS_ZOOM_EVENT_NONE:
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+
+  view_container = gtk_stack_get_visible_child (GTK_STACK (self->stack));
+  view = photos_preview_view_get_view_from_view_container (view_container);
+  photos_image_view_set_zoom (PHOTOS_IMAGE_VIEW (view), zoom, enable_animation);
 
   g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_best_fit_action), TRUE);
   g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_out_action), TRUE);
@@ -902,34 +1096,53 @@ photos_preview_view_zoom_out (PhotosPreviewView *self, GVariant *parameter)
 {
   GtkWidget *view;
   GtkWidget *view_container;
+  PhotosZoomEvent event;
+  gboolean enable_animation;
   gdouble delta;
-  gdouble delta_abs;
   gdouble zoom;
-  gdouble zoom_factor;
-  gdouble zoom_factor_for_delta;
+  gdouble zoom_best_fit;
+  gdouble zoom_best_fit_factor;
 
   g_return_if_fail (self->zoom_best_fit > 0.0);
 
-  delta = g_variant_get_double (parameter);
-  g_return_if_fail (photos_utils_equal_double (delta, -1.0) || delta >= 0.0);
+  event = photos_utils_get_zoom_event (parameter);
+  g_return_if_fail (event != PHOTOS_ZOOM_EVENT_NONE);
+
+  delta = photos_utils_get_zoom_delta (parameter);
+  g_return_if_fail ((event == PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR && photos_utils_equal_double (delta, 1.0))
+                    || (event == PHOTOS_ZOOM_EVENT_MOUSE_CLICK && photos_utils_equal_double (delta, 1.0))
+                    || (event == PHOTOS_ZOOM_EVENT_SCROLL && delta >= 0.0)
+                    || (event == PHOTOS_ZOOM_EVENT_TOUCH && delta >= 0.0 && self->zoom_begin > 0.0));
+
+  switch (event)
+    {
+    case PHOTOS_ZOOM_EVENT_KEYBOARD_ACCELERATOR:
+    case PHOTOS_ZOOM_EVENT_MOUSE_CLICK:
+    case PHOTOS_ZOOM_EVENT_SCROLL:
+      enable_animation = TRUE;
+      zoom = photos_preview_view_calculate_zoom_for_non_touch (self, delta, event, FALSE);
+      zoom_best_fit_factor = 1.0;
+      break;
+
+    case PHOTOS_ZOOM_EVENT_TOUCH:
+      enable_animation = FALSE;
+      zoom = self->zoom_begin * delta;
+      zoom_best_fit_factor = ZOOM_BEST_FIT_FACTOR_TOUCH;
+      break;
+
+    case PHOTOS_ZOOM_EVENT_NONE:
+    default:
+      g_assert_not_reached ();
+      break;
+    }
 
   view_container = gtk_stack_get_visible_child (GTK_STACK (self->stack));
   view = photos_preview_view_get_view_from_view_container (view_container);
 
-  if (delta >= 0.0)
-    zoom_factor = ZOOM_FACTOR_3;
-  else
-    zoom_factor = ZOOM_FACTOR_2;
-
-  delta_abs = fabs (delta);
-  zoom_factor_for_delta = pow (zoom_factor, delta_abs);
-
-  zoom = photos_image_view_get_zoom (PHOTOS_IMAGE_VIEW (view));
-  zoom /= zoom_factor_for_delta;
-
-  if (zoom < self->zoom_best_fit || photos_utils_equal_double (self->zoom_best_fit, zoom))
+  zoom_best_fit = self->zoom_best_fit * zoom_best_fit_factor;
+  if (zoom < zoom_best_fit || photos_utils_equal_double (zoom_best_fit, zoom))
     {
-      photos_image_view_set_best_fit (PHOTOS_IMAGE_VIEW (view), TRUE);
+      photos_image_view_set_best_fit (PHOTOS_IMAGE_VIEW (view), TRUE, TRUE);
       g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_best_fit_action), FALSE);
       g_simple_action_set_enabled (G_SIMPLE_ACTION (self->zoom_out_action), FALSE);
       photos_preview_nav_buttons_set_auto_hide (self->nav_buttons, TRUE);
@@ -937,7 +1150,7 @@ photos_preview_view_zoom_out (PhotosPreviewView *self, GVariant *parameter)
     }
   else
     {
-      photos_image_view_set_zoom (PHOTOS_IMAGE_VIEW (view), zoom);
+      photos_image_view_set_zoom (PHOTOS_IMAGE_VIEW (view), zoom, enable_animation);
     }
 }
 
@@ -955,6 +1168,7 @@ photos_preview_view_dispose (GObject *object)
 
   g_clear_object (&self->node);
   g_clear_object (&self->item_mngr);
+  g_clear_object (&self->gesture_zoom);
   g_clear_object (&self->mode_cntrlr);
   g_clear_object (&self->nav_buttons);
 
@@ -978,14 +1192,38 @@ static void
 photos_preview_view_constructed (GObject *object)
 {
   PhotosPreviewView *self = PHOTOS_PREVIEW_VIEW (object);
+  GtkGesture *gesture_zoom;
 
   G_OBJECT_CLASS (photos_preview_view_parent_class)->constructed (object);
 
+  gesture_zoom = gtk_gesture_zoom_new (GTK_WIDGET (self));
+  gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (gesture_zoom), GTK_PHASE_CAPTURE);
+  g_signal_connect_swapped (gesture_zoom,
+                            "begin",
+                            G_CALLBACK (photos_preview_view_gesture_zoom_begin),
+                            self);
+  g_signal_connect_swapped (gesture_zoom,
+                            "end",
+                            G_CALLBACK (photos_preview_view_gesture_zoom_end),
+                            self);
+
+  self->gesture_zoom = photos_gesture_zoom_new (gesture_zoom);
+  g_signal_connect_swapped (self->gesture_zoom,
+                            "direction-changed",
+                            G_CALLBACK (photos_preview_view_gesture_zoom_direction_changed),
+                            self);
+  g_signal_connect_swapped (self->gesture_zoom,
+                            "scale-changed",
+                            G_CALLBACK (photos_preview_view_gesture_zoom_scale_changed),
+                            self);
+
   self->nav_buttons = photos_preview_nav_buttons_new (self, GTK_OVERLAY (self->overlay));
   g_signal_connect_swapped (self->nav_buttons, "load-next", G_CALLBACK (photos_preview_view_navigate_next), self);
   g_signal_connect_swapped (self->nav_buttons, "load-previous", G_CALLBACK (photos_preview_view_navigate_previous), self);
 
   gtk_widget_show_all (GTK_WIDGET (self));
+
+  g_object_unref (gesture_zoom);
 }
 
 
@@ -1099,6 +1337,13 @@ photos_preview_view_init (PhotosPreviewView *self)
   action = g_action_map_lookup_action (G_ACTION_MAP (app), "sharpen-current");
   g_signal_connect_object (action, "activate", G_CALLBACK (photos_preview_view_sharpen), self, G_CONNECT_SWAPPED);
 
+  self->zoom_begin_action = g_action_map_lookup_action (G_ACTION_MAP (app), "zoom-begin");
+  g_signal_connect_object (self->zoom_begin_action,
+                           "activate",
+                           G_CALLBACK (photos_preview_view_zoom_begin),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   self->zoom_best_fit_action = g_action_map_lookup_action (G_ACTION_MAP (app), "zoom-best-fit");
   g_signal_connect_object (self->zoom_best_fit_action,
                            "activate",
@@ -1106,6 +1351,13 @@ photos_preview_view_init (PhotosPreviewView *self)
                            self,
                            G_CONNECT_SWAPPED);
 
+  self->zoom_end_action = g_action_map_lookup_action (G_ACTION_MAP (app), "zoom-end");
+  g_signal_connect_object (self->zoom_end_action,
+                           "activate",
+                           G_CALLBACK (photos_preview_view_zoom_end),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   self->zoom_in_action = g_action_map_lookup_action (G_ACTION_MAP (app), "zoom-in");
   g_signal_connect_object (self->zoom_in_action,
                            "activate",
