@@ -22,6 +22,7 @@
  *   + Eye of GNOME
  */
 
+
 #include "config.h"
 
 #include <stdio.h>
@@ -46,12 +47,6 @@ struct _PhotosPrintOperation
   gdouble scale_factor;
 };
 
-struct _PhotosPrintOperationClass
-{
-  GtkPrintOperationClass parent_class;
-};
-
-
 enum
 {
   PROP_0,
