@@ -59,14 +59,6 @@ struct _PhotosPropertiesDialog
   guint title_entry_timeout;
 };
 
-struct _PhotosPropertiesDialogClass
-{
-  GtkDialogClass parent_class;
-
-  /* signals */
-  void (*discard_all_edits) (PhotosPropertiesDialog *self);
-};
-
 enum
 {
   PROP_0,
@@ -513,7 +505,7 @@ photos_properties_dialog_constructed (GObject *object)
   location = photos_base_item_get_location (item);
   if (location != NULL && location[0] != '\0' && G_LIKELY (self->queue != NULL))
     {
-      PhotosQuery *query;
+      PhotosQuery *query = NULL;
 
       self->location_w = gtk_label_new (_("Location"));
       gtk_widget_set_halign (self->location_w, GTK_ALIGN_END);
@@ -523,13 +515,13 @@ photos_properties_dialog_constructed (GObject *object)
 
       query = photos_query_builder_location_query (state, location);
       photos_tracker_queue_select (self->queue,
-                                   query->sparql,
+                                   query,
                                    NULL,
                                    photos_properties_dialog_location_query_executed,
                                    g_object_ref (self),
                                    g_object_unref);
 
-      photos_query_free (query);
+      g_object_unref (query);
     }
 
   equipment = photos_base_item_get_equipment (item);
@@ -880,7 +872,7 @@ photos_properties_dialog_class_init (PhotosPropertiesDialogClass *class)
   signals[DISCARD_ALL_EDITS] = g_signal_new ("discard-all-edits",
                                              G_TYPE_FROM_CLASS (class),
                                              G_SIGNAL_RUN_LAST,
-                                             G_STRUCT_OFFSET (PhotosPropertiesDialogClass, discard_all_edits),
+                                             0,
                                              NULL, /* accumulator */
                                              NULL, /* accu_data */
                                              g_cclosure_marshal_VOID__VOID,
