@@ -1,5 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
+ * Copyright © 2017 Red Hat, Inc.
  * Copyright © 2016 Umang Jain
  *
  * This program is free software; you can redistribute it and/or
@@ -31,6 +32,10 @@ G_BEGIN_DECLS
 #define PHOTOS_TYPE_SHARE_NOTIFICATION (photos_share_notification_get_type ())
 G_DECLARE_FINAL_TYPE (PhotosShareNotification, photos_share_notification, PHOTOS, SHARE_NOTIFICATION, GtkGrid);
 
+void                photos_share_notification_new             (PhotosSharePoint *share_point,
+                                                               PhotosBaseItem *item,
+                                                               const gchar *uri);
+
 void                photos_share_notification_new_with_error  (PhotosSharePoint *share_point, GError *error);
 
 G_END_DECLS
