@@ -32,6 +32,7 @@
 #include "photos-tracker-change-monitor.h"
 #include "photos-tracker-queue.h"
 #include "photos-tracker-resources.h"
+#include "photos-query.h"
 
 
 struct _PhotosTrackerChangeMonitor
@@ -46,14 +47,6 @@ struct _PhotosTrackerChangeMonitor
   guint pending_events_id;
 };
 
-struct _PhotosTrackerChangeMonitorClass
-{
-  GObjectClass parent_class;
-
-  /* signals */
-  void (*changes_pending) (PhotosTrackerChangeMonitor *self, GHashTable *changes);
-};
-
 enum
 {
   CHANGES_PENDING,
@@ -258,6 +251,7 @@ photos_tracker_change_monitor_process_events (PhotosTrackerChangeMonitor *self)
   GQueue *events;
   GString *sparql;
   PhotosTrackerChangeMonitorQueryData *data;
+  PhotosQuery *query = NULL;
   gpointer id;
 
   events = self->pending_events;
@@ -276,15 +270,18 @@ photos_tracker_change_monitor_process_events (PhotosTrackerChangeMonitor *self)
 
   g_string_append (sparql, " {}");
 
+  query = photos_query_new (NULL, sparql->str);
+
   data = photos_tracker_change_monitor_query_data_new (self, id_table, events);
   photos_tracker_queue_select (self->queue,
-                               sparql->str,
+                               query,
                                NULL,
                                photos_tracker_change_monitor_query_executed,
                                data,
                                NULL);
 
   g_string_free (sparql, TRUE);
+  g_object_unref (query);
   return G_SOURCE_REMOVE;
 }
 
@@ -413,7 +410,7 @@ photos_tracker_change_monitor_class_init (PhotosTrackerChangeMonitorClass *class
   signals[CHANGES_PENDING] = g_signal_new ("changes-pending",
                                            G_TYPE_FROM_CLASS (class),
                                            G_SIGNAL_RUN_LAST,
-                                           G_STRUCT_OFFSET (PhotosTrackerChangeMonitorClass, changes_pending),
+                                           0,
                                            NULL, /*accumulator */
                                            NULL, /*accu_data */
                                            g_cclosure_marshal_VOID__BOXED,
