@@ -35,6 +35,7 @@
 #include <libgd/gd.h>
 
 #include "photos-application.h"
+#include "photos-enums.h"
 #include "photos-error.h"
 #include "photos-facebook-item.h"
 #include "photos-flickr-item.h"
@@ -42,6 +43,7 @@
 #include "photos-google-item.h"
 #include "photos-local-item.h"
 #include "photos-media-server-item.h"
+#include "photos-offset-collection-view-controller.h"
 #include "photos-offset-collections-controller.h"
 #include "photos-offset-favorites-controller.h"
 #include "photos-offset-overview-controller.h"
@@ -58,6 +60,7 @@
 #include "photos-tool-crop.h"
 #include "photos-tool-enhance.h"
 #include "photos-tool-filters.h"
+#include "photos-tracker-collection-view-controller.h"
 #include "photos-tracker-collections-controller.h"
 #include "photos-tracker-favorites-controller.h"
 #include "photos-tracker-overview-controller.h"
@@ -392,6 +395,41 @@ photos_utils_create_thumbnail (GFile *file,
 }
 
 
+GVariant *
+photos_utils_create_zoom_target_value (gdouble delta, PhotosZoomEvent event)
+{
+  GEnumClass *zoom_event_class = NULL;
+  GEnumValue *event_value;
+  GVariant *delta_value;
+  GVariant *event_nick_value;
+  GVariant *ret_val = NULL;
+  GVariantBuilder builder;
+  const gchar *event_nick = "none";
+
+  g_return_val_if_fail (delta >= 0.0, NULL);
+  g_return_val_if_fail (event != PHOTOS_ZOOM_EVENT_NONE, NULL);
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
+
+  delta_value = g_variant_new_double (delta);
+  g_variant_builder_add (&builder, "{sv}", "delta", delta_value);
+
+  zoom_event_class = G_ENUM_CLASS (g_type_class_ref (PHOTOS_TYPE_ZOOM_EVENT));
+
+  event_value = g_enum_get_value (zoom_event_class, (gint) event);
+  if (event_value != NULL)
+    event_nick = event_value->value_nick;
+
+  event_nick_value = g_variant_new_string (event_nick);
+  g_variant_builder_add (&builder, "{sv}", "event", event_nick_value);
+
+  ret_val = g_variant_builder_end (&builder);
+
+  g_type_class_unref (zoom_event_class);
+  return ret_val;
+}
+
+
 static GIcon *
 photos_utils_get_thumbnail_icon (const gchar *uri)
 {
@@ -418,9 +456,6 @@ photos_utils_get_thumbnail_icon (const gchar *uri)
     }
 
   thumb_path = g_file_info_get_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH);
-  if (thumb_path == NULL)
-    goto out;
-
   thumb_file = g_file_new_for_path (thumb_path);
   icon = g_file_icon_new (thumb_file);
 
@@ -470,7 +505,7 @@ photos_utils_get_icon_from_cursor (TrackerSparqlCursor *cursor)
     goto out;
 
   rdf_type = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_RDF_TYPE, NULL);
-  if (rdf_type != NULL)
+  if (mime_type != NULL)
     icon = photos_utils_icon_from_rdf_type (rdf_type);
 
   if (icon != NULL)
@@ -483,6 +518,57 @@ photos_utils_get_icon_from_cursor (TrackerSparqlCursor *cursor)
 }
 
 
+gdouble
+photos_utils_get_zoom_delta (GVariant *dictionary)
+{
+  gdouble delta;
+  gdouble ret_val = -1.0;
+
+  g_return_val_if_fail (dictionary != NULL, -1.0);
+  g_return_val_if_fail (g_variant_is_of_type (dictionary, G_VARIANT_TYPE_VARDICT), -1.0);
+
+  if (!g_variant_lookup (dictionary, "delta", "d", &delta))
+    goto out;
+
+  ret_val = delta;
+
+ out:
+  g_return_val_if_fail (ret_val >= 0.0, -1.0);
+  return ret_val;
+}
+
+
+PhotosZoomEvent
+photos_utils_get_zoom_event (GVariant *dictionary)
+{
+  GEnumClass *zoom_event_class = NULL;
+  GEnumValue *event_value;
+  PhotosZoomEvent ret_val = PHOTOS_ZOOM_EVENT_NONE;
+  const gchar *event_str;
+
+  g_return_val_if_fail (dictionary != NULL, PHOTOS_ZOOM_EVENT_NONE);
+  g_return_val_if_fail (g_variant_is_of_type (dictionary, G_VARIANT_TYPE_VARDICT), PHOTOS_ZOOM_EVENT_NONE);
+
+  if (!g_variant_lookup (dictionary, "event", "&s", &event_str))
+    goto out;
+
+  zoom_event_class = G_ENUM_CLASS (g_type_class_ref (PHOTOS_TYPE_ZOOM_EVENT));
+
+  event_value = g_enum_get_value_by_nick (zoom_event_class, event_str);
+  if (event_value == NULL)
+    event_value = g_enum_get_value_by_name (zoom_event_class, event_str);
+  if (event_value == NULL)
+    goto out;
+
+  ret_val = (PhotosZoomEvent) event_value->value;
+
+ out:
+  g_clear_pointer (&zoom_event_class, g_type_class_unref);
+  g_return_val_if_fail (ret_val != PHOTOS_ZOOM_EVENT_NONE, PHOTOS_ZOOM_EVENT_NONE);
+  return ret_val;
+}
+
+
 GdkPixbuf *
 photos_utils_downscale_pixbuf_for_scale (GdkPixbuf *pixbuf, gint size, gint scale)
 {
@@ -643,6 +729,7 @@ photos_utils_ensure_builtins (void)
       g_type_ensure (PHOTOS_TYPE_TOOL_ENHANCE);
       g_type_ensure (PHOTOS_TYPE_TOOL_FILTERS);
 
+      g_type_ensure (PHOTOS_TYPE_TRACKER_COLLECTION_VIEW_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_COLLECTIONS_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_FAVORITES_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_OVERVIEW_CONTROLLER);
@@ -935,6 +1022,11 @@ photos_utils_get_controller (PhotosWindowMode mode,
 
   switch (mode)
     {
+    case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
+      offset_cntrlr = photos_offset_collection_view_controller_dup_singleton ();
+      trk_cntrlr = photos_tracker_collection_view_controller_dup_singleton ();
+      break;
+
     case PHOTOS_WINDOW_MODE_COLLECTIONS:
       offset_cntrlr = photos_offset_collections_controller_dup_singleton ();
       trk_cntrlr = photos_tracker_collections_controller_dup_singleton ();
@@ -1220,6 +1312,26 @@ photos_utils_object_list_free_full (GList *objects)
 }
 
 
+gchar *
+photos_utils_print_zoom_action_detailed_name (const gchar *action_name, gdouble delta, PhotosZoomEvent event)
+{
+  GVariant *target_value;
+  gchar *ret_val = NULL;
+
+  g_return_val_if_fail (action_name != NULL && action_name[0] != '\0', NULL);
+  g_return_val_if_fail (delta >= 0.0, NULL);
+  g_return_val_if_fail (event != PHOTOS_ZOOM_EVENT_NONE, NULL);
+
+  target_value = photos_utils_create_zoom_target_value (delta, event);
+  target_value = g_variant_ref_sink (target_value);
+
+  ret_val = g_action_print_detailed_name (action_name, target_value);
+
+  g_variant_unref (target_value);
+  return ret_val;
+}
+
+
 static gboolean
 photos_utils_adjustment_can_scroll (GtkAdjustment *adjustment)
 {
@@ -1310,10 +1422,12 @@ void
 photos_utils_set_edited_name (const gchar *urn, const gchar *title)
 {
   GError *error;
+  PhotosQuery *query = NULL;
   PhotosTrackerQueue *queue = NULL;
   gchar *sparql = NULL;
 
   sparql = g_strdup_printf ("INSERT OR REPLACE { <%s> nie:title \"%s\" }", urn, title);
+  query = photos_query_new (NULL, sparql);
 
   error = NULL;
   queue = photos_tracker_queue_dup_singleton (NULL, &error);
@@ -1324,9 +1438,10 @@ photos_utils_set_edited_name (const gchar *urn, const gchar *title)
       goto out;
     }
 
-  photos_tracker_queue_update (queue, sparql, NULL, photos_utils_update_executed, g_strdup (urn), g_free);
+  photos_tracker_queue_update (queue, query, NULL, photos_utils_update_executed, g_strdup (urn), g_free);
 
  out:
+  g_clear_object (&query);
   g_clear_object (&queue);
   g_free (sparql);
 }
@@ -1336,12 +1451,14 @@ void
 photos_utils_set_favorite (const gchar *urn, gboolean is_favorite)
 {
   GError *error;
+  PhotosQuery *query = NULL;
   PhotosTrackerQueue *queue = NULL;
   gchar *sparql = NULL;
 
   sparql = g_strdup_printf ("%s { <%s> nao:hasTag nao:predefined-tag-favorite }",
                             (is_favorite) ? "INSERT OR REPLACE" : "DELETE",
                             urn);
+  query = photos_query_new (NULL, sparql);
 
   error = NULL;
   queue = photos_tracker_queue_dup_singleton (NULL, &error);
@@ -1352,10 +1469,11 @@ photos_utils_set_favorite (const gchar *urn, gboolean is_favorite)
       goto out;
     }
 
-  photos_tracker_queue_update (queue, sparql, NULL, photos_utils_update_executed, g_strdup (urn), g_free);
+  photos_tracker_queue_update (queue, query, NULL, photos_utils_update_executed, g_strdup (urn), g_free);
 
  out:
   g_free (sparql);
+  g_clear_object (&query);
   g_clear_object (&queue);
 }
 
