@@ -25,6 +25,7 @@
 
 #include <gio/gio.h>
 
+#include "photos-utils.h"
 #include "photos-zoom-controls.h"
 
 
@@ -74,7 +75,12 @@ photos_zoom_controls_update_buttons (PhotosZoomControls *self)
     }
   else
     {
-      gtk_actionable_set_action_target (GTK_ACTIONABLE (self->zoom_toggle_button), "d", -1.0);
+      GVariant *target_value = NULL;
+
+      target_value = photos_utils_create_zoom_target_value (1.0, PHOTOS_ZOOM_EVENT_MOUSE_CLICK);
+      g_assert_true (g_variant_is_floating (target_value));
+
+      gtk_actionable_set_action_target_value (GTK_ACTIONABLE (self->zoom_toggle_button), target_value);
       gtk_actionable_set_action_name (GTK_ACTIONABLE (self->zoom_toggle_button), "app.zoom-in");
     }
 
